// Copyright (C) 2017 Keelan Lightfoot
// Copyright (C) 2007-2008 Board of Regents of the University of Wisconsin
//             System (Univ. of Wisconsin-Madison, Trace R&D Center)
// Copyright (C) 2007-2008 Omnitor AB
// Copyright (C) 2007-2008 Voiceriver Inc
//
// This library is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or (at
// your option) any later version.

// Package gobaudot provides a set of tools to encode, modulate, demodulate and
// baudot/TTY/RTTY data streams
package gortty

import "math"

const (
	// EventDemodChar is fun
	EventDemodChar = iota
	// EventDemodAbort is fun
	EventDemodAbort
	// EventDemodCase inform application regarding a demodulated LETRS/Figures case character
	EventDemodCase

	// EventActive is triggered when the modulator exits out of an idle period to begin modulating
	EventActive
	// EventIdle is triggered after idle
	EventIdle

	// EventStartBit is triggered with the falling edge of the start bit when transmitting a character
	EventStartBit
	// EventStopBit is triggered with rising edge of the stop bit when transmitting a character
	EventStopBit

	// EventEncodeStringComplete is fun
	EventEncodeStringComplete
)

// Callback is func
type Callback func(event int, data interface{})

// ModemSettings is fun
type ModemSettings struct {
	Baud       float64
	StopBits   float64
	DataBits   int
	SampleRate int
	OneFreq    int
	ZeroFreq   int
}

func multq15(x, y int16) int16 {
	return int16((int(x) * int(y)) >> 15)
}

func abs16(x int16) int16 {
	if x < 0 {
		return -x
	}
	return x
}

func absint(x int) int {
	if x < 0 {
		return -x
	}
	return x
}

func calculateW(oneF int, zeroF int, sampleRate int) (float64, float64) {
	wOne := (2 * math.Pi * float64(oneF) / float64(sampleRate))   //1400 // WOne 1 freq in normalised rads
	wZero := (2 * math.Pi * float64(zeroF) / float64(sampleRate)) //1800 // WZero 0 freq in normazlised rads
	return wOne, wZero
}
