// Copyright (C) 2017 Keelan Lightfoot
// Copyright (C) 2007-2008 Board of Regents of the University of Wisconsin
//             System (Univ. of Wisconsin-Madison, Trace R&D Center)
// Copyright (C) 2007-2008 Omnitor AB
// Copyright (C) 2007-2008 Voiceriver Inc
//
// This library is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or (at
// your option) any later version.

package gortty

import (
	"context"
	"unicode/utf8"
)

// Decoder converts a stream of bytes representing baudot codes into a stream
// of unicode runes.
type Decoder struct {
	input   chan byte
	output  chan rune
	charset *CodeSet
	usos    bool
}

// UnshiftOnSpace sets how the decoder behaves when a space is received while in figures
// mode
func (d *Decoder) UnshiftOnSpace(usos bool) {
	d.usos = usos
}

// NewDecoder creates a new decoder.
func NewDecoder(ctx context.Context, input chan byte, output chan rune, charset *CodeSet) *Decoder {
	d := new(Decoder)
	d.input = input
	if output == nil {
		d.output = make(chan rune)
	} else {
		d.output = output
	}
	d.charset = charset

	go d.run(ctx)
	return d
}

func (d *Decoder) run(ctx context.Context) {
	shift := shiftLetter

	for {
		select {
		case <-ctx.Done():
			return
		case c := <-d.input:
			switch c {
			case d.charset.UnshiftCode:
				shift = shiftLetter
				//TODO: Trigger EventDemodCase
			case d.charset.ShiftCode:
				shift = shiftFigure
				//TODO: Trigger EventDemodCase
			default:
				r := d.charset.toRune(shift, c)
				if d.usos && shift == shiftFigure && r == ' ' {
					shift = shiftLetter
					//TODO: Trigger EventUnshiftOnSpace
				}
				d.output <- d.charset.toRune(shift, c)
				//TODO: Trigger EventDemodChar
			}
		}
	}
}

func (d *Decoder) Read(p []byte) (n int, err error) {
	r := <-d.output
	l := utf8.EncodeRune(p, r)
	return l, nil
}
